//Added by qt3to4:
//#include <QEvent>
//#include <Q3PtrList>
// Copyright (c) 2000-2001 Brad Hughes <bhughes@trolltech.com>
//
// Use, modification and distribution is allowed without limitation,
// warranty, or liability of any kind.
//

#ifndef   DECODER_H
#define   DECODER_H

#include <QThread>
#include <QList>
#include <QMutex>
#include <QWaitCondition>
#include <QObject>


#include "fileinfo.h"
#include "filetag.h"

class QObject;
class QIODevice;

class Decoder;
class DecoderFactory;
class Buffer;
class Recycler;
class Output;
class Visual;



class DecoderState
{
public:
    enum Type { Decoding, Stopped, Finished, Error };

    DecoderState(Type t)
            : m_type(t), m_error_msg(0)
    {}

    DecoderState(QString *e)
            : m_type(Error), m_error_msg(e)
    {}
     
    DecoderState()
            : m_type(Stopped), m_error_msg(0)
    {}

    ~DecoderState()
    {
        if (m_error_msg)
           delete m_error_msg;
    }

    const QString *errorMessage() const
    {
        return m_error_msg;
    }
    const Type &type() const
    {
        return m_type;
    }

private:
    Type m_type;
    QString *m_error_msg;
};



class Decoder : public QThread
{
Q_OBJECT
public:
    Decoder(QObject *parent, DecoderFactory *d, 
               QIODevice *i, Output *o);
    virtual ~Decoder();

    // Standard Decoder API
    virtual bool initialize() = 0;
    virtual double lengthInSeconds() = 0;
    virtual void seek(double) = 0;
    virtual void stop() = 0;

    // Equalizer
    /*virtual bool isEQSupported() const = 0;
    virtual void setEQEnabled(bool) = 0;
    virtual void setEQ(const EqPreset &preset)
    {
        eq = preset;
    }

    const EqPreset &EQ() const
    {
        return eq;
    }*/

    DecoderFactory *factory() const
    {
        return fctry;
    }

    void addListener(QObject *);
    void removeListener(QObject *);

    QIODevice *input()
    {
        return in;
    }
    Output *output()
    {
        return out;
    }
    void setInput(QIODevice *);
    void setOutput(Output *);

    QMutex *mutex()
    {
        return &mtx;
    }
    QWaitCondition *cond()
    {
        return &cnd;
    }

    void setBlockSize(unsigned int sz)
    {
        blksize = sz;
    }

    unsigned int blockSize() const
    {
        return blksize;
    }
    ulong produceSound(char *data, ulong output_bytes, ulong bitrate, int nch);
    void setEQ(int bands[10], int preamp);
    void setEQEnabled(bool on) { m_useEQ = on; };

    // static methods
    static QStringList all();
    static bool supports(const QString &);
    static void registerFactory(DecoderFactory *);
    static Decoder *create(QObject *, const QString &, QIODevice *,
                           Output *, bool = FALSE);
    static DecoderFactory *findFactory(const QString&);
    static FileTag *createTag(const QString&);
    static QString filter();
    static QStringList nameFilters();
    static QList<DecoderFactory*> *decoderFactories();
    static QStringList decoderFiles();

signals:
    void stateChanged(DecoderState *);  
  
protected:
    //Decoder(DecoderFactory *, QIODevice *, Output *);

    void dispatch(const DecoderState&);
    void error(const QString&);
    //bool m_eqInited;

private:
    DecoderFactory *fctry;
    DecoderState m_state;

    //Q3PtrList<QObject> listeners;
    QList<QObject*> listeners;
    QIODevice *in;
    Output *out;

    QMutex mtx;
    QWaitCondition cnd;

    uint blksize;
    bool m_eqInited;
    bool m_useEQ;
};




#endif // DECODER_H
