/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QSettings>
#include <QEvent>

#include "skin.h"
#include "eqslider.h"
#include "eqtitlebar.h"
#include "togglebutton.h"
#include "eqgraph.h"
#include "eqwidget.h"



EqWidget::EqWidget ( QWidget *parent )
      : PixmapWidget ( parent )
{
   m_skin = Skin::getPointer();
   setWindowFlags ( Qt::Dialog | Qt::FramelessWindowHint );
   setPixmap ( m_skin->getEqPart ( Skin::EQ_MAIN ) );
   //setPixmap(QPixmap(275,116));
   m_titleBar = new EqTitleBar ( this );
   m_titleBar -> move ( 0,0 );
   m_titleBar -> show();
   connect ( m_skin, SIGNAL ( skinChanged() ), this, SLOT ( updateSkin() ) );

   m_preamp = new EqSlider ( this );
   m_preamp->show();
   m_preamp->move ( 21,38 );
   connect ( m_preamp,SIGNAL ( sliderMoved ( int ) ),SLOT ( setPreamp ( int ) ) );

   m_on = new ToggleButton ( this,Skin::EQ_BT_ON_N,Skin::EQ_BT_ON_P,
                             Skin::EQ_BT_OFF_N,Skin::EQ_BT_OFF_P );
   m_on->show();
   m_on->move ( 14,18 );
   connect (m_on, SIGNAL (clicked(bool)), SIGNAL(valueChanged()));

   m_eqg = new EQGraph(this);
   m_eqg->move(87,17);
   //widget->setAutoFillBackground ( TRUE );
   m_eqg->show();
   //
   /*QPalette palette;
   palette.setColor(widget->backgroundRole(), "Green");
   widget->setPalette(palette);*/

   for ( int i = 0; i<10; ++i )
   {
      m_sliders << new EqSlider ( this );
      m_sliders.at ( i )->move ( 78+i*18,38 );
      m_sliders.at ( i )->show();
      connect ( m_sliders.at ( i ),SIGNAL ( sliderMoved ( int ) ),SLOT ( setGain() ) );
   }
   readSettings();
}

EqWidget::~EqWidget()
{}

int EqWidget::preamp()
{
   return m_preamp->value();
}

int EqWidget::gain ( int g )
{
   return m_sliders.at ( g )->value();
}

/*void EqWidget::enterEvent ( QEvent * )
{
   m_titleBar->setActive ( TRUE );
}

void EqWidget::leaveEvent ( QEvent * )
{
   m_titleBar->setActive ( FALSE );
}*/

void EqWidget::changeEvent ( QEvent * event )
{
  if (event->type() == QEvent::ActivationChange)
  {
     m_titleBar->setActive(isActiveWindow());
  }
}

void EqWidget::closeEvent ( QCloseEvent* )
{
   writeSettings();
}

void EqWidget::updateSkin()
{
   m_titleBar->setActive ( FALSE );
   setPixmap ( m_skin->getEqPart ( Skin::EQ_MAIN ) );
}

void EqWidget::readSettings()
{
   QSettings settings ( QDir::homePath() +"/.qmmp/qmmprc", QSettings::IniFormat );
   settings.beginGroup ( "Equalizer" );
   //geometry
   move ( settings.value ( "pos", QPoint ( 100, 216 ) ).toPoint() );
   settings.endGroup();
}

void EqWidget::writeSettings()
{
   QSettings settings ( QDir::homePath() +"/.qmmp/qmmprc", QSettings::IniFormat );
   settings.beginGroup ( "Equalizer" );
   //geometry
   settings.setValue ( "pos", this->pos() );
   settings.endGroup();
}

void EqWidget::setPreamp ( int p )
{
   //set_preamp(0, 1.0 + 0.0932470 *p + 0.00279033 * p * p);
   //set_preamp(1, 1.0 + 0.0932470 *p + 0.00279033 * p * p);
   emit valueChanged();
}

void EqWidget::setGain()
{
   //qDebug("setGain()");
   m_eqg->clear();
   for(int i=0; i<10; ++i)
   {
      int value = m_sliders.at(i)->value();
      //qDebug("value %d",value);
      m_eqg->addValue(value);
      //set_gain(i,0, 0.03*value+0.000999999*value*value);
      //set_gain(i,1, 0.03*value+0.000999999*value*value);
   }
   emit valueChanged();
}

bool EqWidget::isEQEnabled()
{
   return m_on->isChecked();
}
