/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                       *
 *   forkotov02@hotmail.ru                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QPainter>
#include <QResizeEvent>

#include "skin.h"
#include "playlistslider.h"
#include "pixmapwidget.h"

PlayListSlider::PlayListSlider(QWidget *parent)
        : QWidget(parent)
{
    m_skin = Skin::getPointer();
    old_sy = (height()-58)/25;

    m_scroll = new PixmapWidget(this);
    m_scroll->setPixmap(m_skin->getButton(Skin::PL_BT_SCROLL_N));
    m_scroll->move(5,0);

    m_moving = FALSE;
    m_max = 99;
    pos = 0;
    connect(m_skin, SIGNAL(skinChanged()), this, SLOT(updateSkin()));
}


PlayListSlider::~PlayListSlider()
{}

void PlayListSlider::paintEvent(QPaintEvent *)
{
    int sy = (height()-58)/29;
    QPainter paint;
    paint.begin(this);
    paint.drawPixmap(0,0,m_skin->getPlPart(Skin::PL_RFILL));
    paint.drawPixmap(0,29,m_skin->getPlPart(Skin::PL_RFILL));

    for (int i = 0; i<sy; i++)
    {
        paint.drawPixmap(0,58+i*29,m_skin->getPlPart(Skin::PL_RFILL));
    }

    paint.end();
}

void PlayListSlider::resizeEvent(QResizeEvent *)
{
    //int sy = 58+(height ()) *25;


    /*if (sy == old_sy)
        return;
    old_sy = sy;*/

    setUpdatesEnabled(false);
    int l = height() - m_scroll->height();
    m_scroll->move(5, int(l*pos/m_max));
    setUpdatesEnabled(true);
}

void PlayListSlider::mousePressEvent(QMouseEvent *e)
{

    m_moving = TRUE;
    if (!m_scroll->testAttribute(Qt::WA_UnderMouse))
    {
        m_scroll->move(5, e->y()-m_scroll->height()/2);
    }
    m_scroll->setPixmap(m_skin->getButton(Skin::PL_BT_SCROLL_P));
    press_pos =  e->y() - m_scroll->y();
}

void PlayListSlider::mouseReleaseEvent(QMouseEvent*)
{
    m_moving = FALSE;
    m_scroll->setPixmap(m_skin->getButton(Skin::PL_BT_SCROLL_N));
    //int po = m_barButton->x();
    //int l = width() - m_barButton->width();
    //mw->seek(int(po*m_max/l));

}

void PlayListSlider::mouseMoveEvent(QMouseEvent* e)
{
    if(m_moving)
    {
        int po = e->y();
        po = po - press_pos;
        int l = height() - m_scroll->height();
        if( (po >= 0) && (po <= l) )
        {
            //m_scroll->move(5, po);
            //int l = height() - m_scroll->height();

            //if (po%(l/m_max) == 0)
            m_scroll->move(5, po);
            pos = po*m_max/l;
            emit sliderMoved(pos);

        }

    }
}

void PlayListSlider::setPos(int p)
{
    if(m_moving)
        return;
    pos = p;

    int l = height() - m_scroll->height();
    m_scroll->move(5, int(l*p/m_max));
}

void PlayListSlider::setMax(long m)
{
    m_max = m;
}

void PlayListSlider::updateSkin()
{
    update();
    m_scroll->setPixmap(m_skin->getButton(Skin::PL_BT_SCROLL_N));
}
